==> firefox-8.0.source.tar.bz2: mozilla-release/intl/locale/src/nsLanguageAtomService.cpp <==
/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is mozilla.org code.
 *
 * The Initial Developer of the Original Code is
 * Netscape Communications Corporation.
 * Portions created by the Initial Developer are Copyright (C) 2000
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either of the GNU General Public License Version 2 or later (the "GPL"),
 * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

#include "nsIComponentManager.h"
#include "nsLanguageAtomService.h"
#include "nsICharsetConverterManager.h"
#include "nsILocaleService.h"
#include "nsXPIDLString.h"
#include "nsUnicharUtils.h"
#include "nsIServiceManager.h"
#include "nsIAtom.h"
#include "mozilla/Services.h"

NS_IMPL_ISUPPORTS1(nsLanguageAtomService, nsILanguageAtomService)

nsLanguageAtomService::nsLanguageAtomService()
{
  mLangToGroup.Init();
}

nsresult
nsLanguageAtomService::InitLangGroupTable()
{
  if (mLangGroups)
    return NS_OK;

  nsCOMPtr<nsIStringBundleService> bundleService =
    mozilla::services::GetStringBundleService();
  if (!bundleService)
    return NS_ERROR_FAILURE;

  return bundleService->CreateBundle("resource://gre/res/langGroups.properties",
                                     getter_AddRefs(mLangGroups));
}

nsIAtom*
nsLanguageAtomService::LookupLanguage(const nsACString &aLanguage,
                                      nsresult *aError)
{
  nsCAutoString lowered(aLanguage);
  ToLowerCase(lowered);

  nsCOMPtr<nsIAtom> lang = do_GetAtom(lowered);
  return GetLanguageGroup(lang, aError);
}

already_AddRefed<nsIAtom>
nsLanguageAtomService::LookupCharSet(const char *aCharSet, nsresult *aError)
{
  if (!mCharSets) {
    mCharSets = do_GetService(NS_CHARSETCONVERTERMANAGER_CONTRACTID);
    if (!mCharSets) {
      if (aError)
        *aError = NS_ERROR_FAILURE;

      return nsnull;
    }
  }

  nsCOMPtr<nsIAtom> langGroup;
  mCharSets->GetCharsetLangGroup(aCharSet, getter_AddRefs(langGroup));
  if (!langGroup) {
    if (aError)
      *aError = NS_ERROR_FAILURE;

    return nsnull;
  }

  // transfer reference to raw pointer
  nsIAtom *raw = nsnull;
  langGroup.swap(raw);

  if (aError)
    *aError = NS_OK;

  return raw;
}

nsIAtom*
nsLanguageAtomService::GetLocaleLanguage(nsresult *aError)
{
  nsresult res = NS_OK;

  do {
    if (!mLocaleLanguage) {
      nsCOMPtr<nsILocaleService> localeService;
      localeService = do_GetService(NS_LOCALESERVICE_CONTRACTID);
      if (!localeService) {
        res = NS_ERROR_FAILURE;
        break;
      }

      nsCOMPtr<nsILocale> locale;
      res = localeService->GetApplicationLocale(getter_AddRefs(locale));
      if (NS_FAILED(res))
        break;

      nsAutoString category;
      category.AssignWithConversion(NSILOCALE_MESSAGE);
      nsAutoString loc;
      res = locale->GetCategory(category, loc);
      if (NS_FAILED(res))
        break;

      ToLowerCase(loc); // use lowercase for all language atoms
      mLocaleLanguage = do_GetAtom(loc);
    }
  } while (0);

  if (aError)
    *aError = res;

  return mLocaleLanguage;
}

nsIAtom*
nsLanguageAtomService::GetLanguageGroup(nsIAtom *aLanguage,
                                        nsresult *aError)
{
  nsIAtom *retVal;
  nsresult res = NS_OK;

  retVal = mLangToGroup.GetWeak(aLanguage);

  if (!retVal) {
    if (!mLangGroups) {
      if (NS_FAILED(InitLangGroupTable())) {
        if (aError) {
          *aError = NS_ERROR_FAILURE;
        }
        return nsnull;
      }
    }

    nsString langStr;
    aLanguage->ToString(langStr);

    nsXPIDLString langGroupStr;
    res = mLangGroups->GetStringFromName(langStr.get(),
                                         getter_Copies(langGroupStr));
    if (NS_FAILED(res)) {
      PRInt32 hyphen = langStr.FindChar('-');
      if (hyphen >= 0) {
        nsAutoString truncated(langStr);
        truncated.Truncate(hyphen);
        res = mLangGroups->GetStringFromName(truncated.get(),
                                             getter_Copies(langGroupStr));
        if (NS_FAILED(res)) {
          langGroupStr.AssignLiteral("x-unicode");
        }
      } else {
        langGroupStr.AssignLiteral("x-unicode");
      }
    }

    nsCOMPtr<nsIAtom> langGroup = do_GetAtom(langGroupStr);

    // The hashtable will keep an owning reference to the atom
    mLangToGroup.Put(aLanguage, langGroup);
    retVal = langGroup.get();
  }

  if (aError) {
    *aError = res;
  }

  return retVal;
}

==> firefox-16.0.2.source.tar.bz2: mozilla-release/intl/locale/src/nsLanguageAtomService.cpp <==
/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsIComponentManager.h"
#include "nsLanguageAtomService.h"
#include "nsICharsetConverterManager.h"
#include "nsILocaleService.h"
#include "nsXPIDLString.h"
#include "nsUnicharUtils.h"
#include "nsIServiceManager.h"
#include "nsIAtom.h"
#include "mozilla/Services.h"

NS_IMPL_ISUPPORTS1(nsLanguageAtomService, nsILanguageAtomService)

nsLanguageAtomService::nsLanguageAtomService()
{
  mLangToGroup.Init();
}

nsresult
nsLanguageAtomService::InitLangGroupTable()
{
  if (mLangGroups)
    return NS_OK;

  nsCOMPtr<nsIStringBundleService> bundleService =
    mozilla::services::GetStringBundleService();
  if (!bundleService)
    return NS_ERROR_FAILURE;

  return bundleService->CreateBundle("resource://gre/res/langGroups.properties",
                                     getter_AddRefs(mLangGroups));
}

nsIAtom*
nsLanguageAtomService::LookupLanguage(const nsACString &aLanguage,
                                      nsresult *aError)
{
  nsCAutoString lowered(aLanguage);
  ToLowerCase(lowered);

  nsCOMPtr<nsIAtom> lang = do_GetAtom(lowered);
  return GetLanguageGroup(lang, aError);
}

already_AddRefed<nsIAtom>
nsLanguageAtomService::LookupCharSet(const char *aCharSet, nsresult *aError)
{
  if (!mCharSets) {
    mCharSets = do_GetService(NS_CHARSETCONVERTERMANAGER_CONTRACTID);
    if (!mCharSets) {
      if (aError)
        *aError = NS_ERROR_FAILURE;

      return nsnull;
    }
  }

  nsCOMPtr<nsIAtom> langGroup;
  mCharSets->GetCharsetLangGroup(aCharSet, getter_AddRefs(langGroup));
  if (!langGroup) {
    if (aError)
      *aError = NS_ERROR_FAILURE;

    return nsnull;
  }

  // transfer reference to raw pointer
  nsIAtom *raw = nsnull;
  langGroup.swap(raw);

  if (aError)
    *aError = NS_OK;

  return raw;
}

nsIAtom*
nsLanguageAtomService::GetLocaleLanguage(nsresult *aError)
{
  nsresult res = NS_OK;

  do {
    if (!mLocaleLanguage) {
      nsCOMPtr<nsILocaleService> localeService;
      localeService = do_GetService(NS_LOCALESERVICE_CONTRACTID);
      if (!localeService) {
        res = NS_ERROR_FAILURE;
        break;
      }

      nsCOMPtr<nsILocale> locale;
      res = localeService->GetApplicationLocale(getter_AddRefs(locale));
      if (NS_FAILED(res))
        break;

      nsAutoString loc;
      res = locale->GetCategory(NS_LITERAL_STRING(NSILOCALE_MESSAGE), loc);
      if (NS_FAILED(res))
        break;

      ToLowerCase(loc); // use lowercase for all language atoms
      mLocaleLanguage = do_GetAtom(loc);
    }
  } while (0);

  if (aError)
    *aError = res;

  return mLocaleLanguage;
}

nsIAtom*
nsLanguageAtomService::GetLanguageGroup(nsIAtom *aLanguage,
                                        nsresult *aError)
{
  nsIAtom *retVal;
  nsresult res = NS_OK;

  retVal = mLangToGroup.GetWeak(aLanguage);

  if (!retVal) {
    if (!mLangGroups) {
      if (NS_FAILED(InitLangGroupTable())) {
        if (aError) {
          *aError = NS_ERROR_FAILURE;
        }
        return nsnull;
      }
    }

    nsString langStr;
    aLanguage->ToString(langStr);

    nsXPIDLString langGroupStr;
    res = mLangGroups->GetStringFromName(langStr.get(),
                                         getter_Copies(langGroupStr));
    if (NS_FAILED(res)) {
      PRInt32 hyphen = langStr.FindChar('-');
      if (hyphen >= 0) {
        nsAutoString truncated(langStr);
        truncated.Truncate(hyphen);
        res = mLangGroups->GetStringFromName(truncated.get(),
                                             getter_Copies(langGroupStr));
        if (NS_FAILED(res)) {
          langGroupStr.AssignLiteral("x-unicode");
        }
      } else {
        langGroupStr.AssignLiteral("x-unicode");
      }
    }

    nsCOMPtr<nsIAtom> langGroup = do_GetAtom(langGroupStr);

    // The hashtable will keep an owning reference to the atom
    mLangToGroup.Put(aLanguage, langGroup);
    retVal = langGroup.get();
  }

  if (aError) {
    *aError = res;
  }

  return retVal;
}
